# -*- coding: utf-8 -*-
# Copyright (C)2007 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Generic Test case for iw.dist doctest
"""
__docformat__ = 'restructuredtext'

import unittest
import sys
import os

from iw.dist.base import MCommand

class TestMCommand(unittest.TestCase):

    def test_get_pypirc(self):

        wanted = [('acme-login', [('host', 'http://example.org/'), 
                                  ('password', 'secret'), 
                                  ('realm', 'acme'), 
                                  ('username', 'tarek')]), 
                  ('plone-login', [('host', 'http://plone.org/products'), 
                                   ('password', 'secret'), 
                                   ('realm', 'pypi'),
                                   ('username', 'tarek')]), 
                  ('server-login', [('host', 'http://pypi.python.org/pypi'),
                                    ('password', 'secret'), 
                                    ('realm', 'pypi'), 
                                    ('username', 'tarek')])] 
        old_home = os.environ['HOME']
        os.environ['HOME'] = os.path.dirname(__file__)
        try:
            reg = MCommand()
            servers = reg. _get_pypirc()
            servers = servers.items()
            servers.sort()
            def _sort(el):
                key, value = el
                value = value.items()
                value.sort()
                return key, value
            servers = [_sort(el) for el in servers]
            self.assertEquals(servers, wanted)
        finally:
            os.environ['HOME'] = old_home

def test_suite():
    """returns the test suite"""
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMCommand))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# -*- coding: utf-8 -*-
# Copyright (C)2007 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Generic Test case for iw.dist doctest
"""
__docformat__ = 'restructuredtext'

import unittest
import sys
import os

from iw.dist.mupload import mupload
from distutils.core import Distribution 

import httplib

class Response(object):
    status = 200
    reason = 'OK'

class _HTTPConnection(object):
    def __init__(self, netloc):
        pass

    def connect(self):
        pass

    def putrequest(self, *args, **kw):
        pass

    putheader = putrequest
    endheaders = putrequest
    send = putrequest
    
    def getresponse(self):
        return Response()

httplib.HTTPConnection = _HTTPConnection

class TestMUpload(unittest.TestCase):

    def test_finalize_options(self):
        def _raw_input(msg):
            return 'y'
        post_to_server = None
        dist = Distribution()
        old_home = os.environ['HOME']
        os.environ['HOME'] = os.path.dirname(__file__)
        try:
            reg = mupload(dist)
            # patching
            for func_ in (reg.finalize_options, reg.upload_file):
                func_.im_func.func_globals['raw_input'] = _raw_input
            reg.finalize_options()
            path = os.path.join(os.path.dirname(__file__), '.pypirc')
            reg.upload_file('xxx', '', path)
        finally:
            os.environ['HOME'] = old_home

def test_suite():
    """returns the test suite"""
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMUpload))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# -*- coding: utf-8 -*-
# Copyright (C)2007 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
""" mregister

    this command works like register excepts that
    it reads a .pypirc file that supports multiple
    servers
"""
import os
import urllib2
import urlparse
from ConfigParser import ConfigParser

from setuptools.command.register import register
from base import MCommand

class mregister(register, MCommand):

    description = "register the distribution with several indexes"

    def send_metadata(self):
        """allow multiple servers uploads"""
        conf = self._get_pypirc() 
        if conf is None:
            # let the register command do the work
            return register.send_metadata() 
       
        auth = urllib2.HTTPPasswordMgr()
        for server in conf.values():
            host = server['host']
            choice = raw_input(('Do you want to register the package '
                                'metadata at %s (y/N)? ' % host))
            if choice.lower() not in ('y', 'yes'):
                continue
            username = server['username']
            password = server['password']
            realm = server['realm']
            root_host = urlparse.urlparse(host)[1]
            auth.add_password(realm, root_host, username, password)
            
            # XXX this is a hack to avoid rewriting the whole register
            # class
            self.repository = host
            code, result = self.post_to_server(self.build_post_data('submit'),
                                               auth)
            
            print 'Server %s response (%s): %s' % (host, code, result)




# -*- coding: utf-8 -*-
# Copyright (C)2007 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Generic Test case for iw.dist doctest
"""
__docformat__ = 'restructuredtext'

import unittest
import sys
import os

from iw.dist.mregister import mregister
from distutils.core import Distribution 
import urllib2

class Opener(object):
    def open(self, *args, **kw):
        pass

def _build_opener(auth_handler):
    return Opener()

urllib2.build_opener = _build_opener

class TestMRegister(unittest.TestCase):

    def test_send_metadata(self):
        def _raw_input(msg):
            return 'y'
        post_to_server = None
        dist = Distribution()
        old_home = os.environ['HOME']
        os.environ['HOME'] = os.path.dirname(__file__)
        try:
            reg = mregister(dist)
            # patching
            reg.send_metadata.im_func.func_globals['raw_input'] = _raw_input
            reg.send_metadata()
        finally:
            os.environ['HOME'] = old_home



def test_suite():
    """returns the test suite"""
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMRegister))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# -*- coding: utf-8 -*-
# Copyright (C)2007 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
""" mupload

    this command works like upload except that
    it make uploads on several sites
"""
import os
import urlparse
from setuptools.command.upload import upload

from base import MCommand

class mupload(upload, MCommand):

    description = "upload the distribution to several indexes"

    def finalize_options(self): 
        """reading the conf file"""
        self.conf = self._get_pypirc()
        if self.conf is None:
            upload.finalize_options()
        
    def upload_file(self, command, pyversion, filename):
        """upload file on several servers"""
        if self.conf is None:  
            upload.upload_file(self, command, pyversion, filename) 
            return

        for server in self.conf.values():
            host = server['host']
            choice = raw_input(('Do you want to upload the package '
                                'at %s (y/N)? ' % host))
            if choice.lower() not in ('y', 'yes'):
                continue
            
            # XXX hack: setting attributes to avoid rewriting the whole
            # class
            self.repository = host
            self.username = server['username']
            self.password = server['password']
            # XXX not used for now
            #self.realm = server['realm']
            upload.upload_file(self, command, pyversion, filename)         




# -*- coding: utf-8 -*-
# Copyright (C)2007 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
""" provides a .pypirc reader for multiple servers
"""
import os
from ConfigParser import ConfigParser

class MCommand(object):

    DEFAULT_REALM = "pypi"
    DEFAULT_REPOSITORY = "http://pypi.python.org/pypi"

    def _get_pypirc(self):
        """get the pypirc file"""
        if os.environ.has_key('HOME'): 
            rc = os.path.join(os.environ['HOME'], '.pypirc')
            if os.path.exists(rc):
                print 'Using PyPI login from %s' % rc
                config = ConfigParser()
                config.read(rc)
                # make a dict out of the file
                servers = {}
                for section in config.sections():
                    current = {'username': config.get(section, 'username'),
                               'password': config.get(section, 'password')}

                    for key, default in (('host', self.DEFAULT_REPOSITORY),
                                         ('realm', self.DEFAULT_REALM)):
                        if config.has_option(section, key):
                            current[key] = config.get(section, key)
                        else:
                            current[key] = default

                    servers[section] = current
                
                return servers    
        return None

